#include <stdio.h>
#include <time.h>

#include "tinyaes/tinyaes.h"

#define BLOCKSIZE 16
/* 128, 192, 256 bit key length is allowed */
#define KEYLEN 128

int
main(void)
{
	aes_ctx *ctx;
	time_t t;
	static unsigned char key[KEYLEN] = "rijndael128";
	unsigned char buf1[BLOCKSIZE], buf2[BLOCKSIZE];
	unsigned long initcount, encryptcount, decryptcount;

	t = time(NULL);

	/* warm up */
	while (t == time(NULL));
	printf("initialize for 10 seconds\n");
	fflush(stdout);
	initcount = 0;
	while (time(NULL) - t < 10) {
		ctx = aes_init_key(key, KEYLEN);
		aes_delete_key(ctx);
		initcount++;
	}
	printf("%lu keys done.\n", initcount);
	fflush(stdout);

	t = time(NULL);
	/* warm up */
	while (t == time(NULL));
	printf("encrypt for 10 seconds\n");
	fflush(stdout);
	encryptcount = 0;
	ctx = aes_init_key(key, KEYLEN);
	while (time(NULL) - t < 10) {
		aes_ecb_encrypt(ctx, buf2, buf1, BLOCKSIZE);
		encryptcount++;
	}
	encryptcount *= BLOCKSIZE;
	aes_delete_key(ctx);
	printf("%lu bytes done.\n", encryptcount);
	fflush(stdout);

	t = time(NULL);
	/* warm up */
	while (t == time(NULL));
	printf("decrypt for 10 seconds\n");
	fflush(stdout);
	decryptcount = 0;
	ctx = aes_init_key(key, KEYLEN);
	while (time(NULL) - t < 10) {
		aes_ecb_decrypt(ctx, buf2, buf1, BLOCKSIZE);
		decryptcount++;
	}
	decryptcount *= BLOCKSIZE;
	aes_delete_key(ctx);
	printf("%lu bytes done.\n", decryptcount);
	fflush(stdout);

	printf("init:     %12.1f(keys/sec)\n", (double) initcount / 10.0);
	printf("encrypt: %12.1f(bytes/sec)\n", (double) encryptcount / 10.0);
	printf("decrypt: %12.1f(bytes/sec)\n", (double) decryptcount / 10.0);
	return 0;
}
