#ifndef RIJNDAEL_UTIL_H
#define RIJNDAEL_UTIL_H

#include <tinyaes/rijndael-alg-fst.h>

#define MAX_IV_SIZE 16

/* OpenBSD compatible structure */
/*  The structure for key information */
typedef struct {
	int	enc_only;		/* context contains only encrypt schedule */
	int	Nr;			/* key-length-dependent number of rounds */
	u32	ek[4*(MAXNR + 1)];	/* encrypt key schedule */
	u32	dk[4*(MAXNR + 1)];	/* decrypt key schedule */
} aes_ctx;

aes_ctx *aes_init_key(const u8 *, size_t);
void aes_delete_key(aes_ctx *);

int aes_ecb_crypt(aes_ctx *, u8 *, u8 *, size_t, int);
int aes_ecb_crypt_finish(aes_ctx *, u8 *, u8 *, size_t, int);

int aes_cbc_crypt(aes_ctx *, u8 [MAX_IV_SIZE], u8 *, u8 *, size_t, int);
int aes_cbc_crypt_finish(aes_ctx *, u8 [MAX_IV_SIZE], u8 *, u8 *, size_t, int);

int aes_cfb1_crypt(aes_ctx *, u8 [MAX_IV_SIZE], u8 *, u8 *, size_t, int);

#define aes_ecb_encrypt(c, d, s, l) aes_ecb_crypt(c, d, s, l, 1)
#define aes_ecb_encrypt_finish(c, d, s, l) aes_ecb_crypt_finish(c, d, s, l, 1)
#define aes_ecb_decrypt(c, d, s, l) aes_ecb_crypt(c, d, s, l, 0)
#define aes_ecb_decrypt_finish(c, d, s, l) aes_ecb_crypt_finish(c, d, s, l, 0)

#define aes_cbc_encrypt(c, i, d, s, l) aes_cbc_crypt(c, i, d, s, l, 1)
#define aes_cbc_encrypt_finish(c, i, d, s, l) aes_cbc_crypt_finish(c, i, d, s, l, 1)
#define aes_cbc_decrypt(c, i, d, s, l) aes_cbc_crypt(c, i, d, s, l, 0)
#define aes_cbc_decrypt_finish(c, i, d, s, l) aes_cbc_crypt_finish(c, i, d, s, l, 0)

#define aes_cfb1_encrypt(c, i, d, s, l) aes_cfb1_crypt(c, i, d, s, l, 1)
#define aes_cfb1_decrypt(c, i, d, s, l) aes_cfb1_crypt(c, i, d, s, l, 0)


#endif
