/**
 * rijndael-cfb1.c from rijndael-api-fst.c
 *
 * @version 2.9 (December 2000)
 *
 * Optimised ANSI C code for the Rijndael cipher (now AES)
 *
 * @author Vincent Rijmen <vincent.rijmen@esat.kuleuven.ac.be>
 * @author Antoon Bosselaers <antoon.bosselaers@esat.kuleuven.ac.be>
 * @author Paulo Barreto <paulo.barreto@terra.com.br>
 *
 * This code is hereby placed in the public domain.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Acknowledgements:
 *
 * We are deeply indebted to the following people for their bug reports,
 * fixes, and improvement suggestions to this implementation. Though we
 * tried to list all contributions, we apologise in advance for any
 * missing reference.
 *
 * Andrew Bales <Andrew.Bales@Honeywell.com>
 * Markus Friedl <markus.friedl@informatik.uni-erlangen.de>
 * John Skodon <skodonj@webquill.com>
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tinyaes/tinyaes.h"

int
aes_cfb1_crypt(aes_ctx *ctx, u8 IV[MAX_IV_SIZE],
	       u8 *outBuffer, u8 *input, size_t inputLen, int mode)
{
	int i, k, t, numBlocks;
	u8 block[16], *iv;
	u8 *p;
	u8 mod;

	numBlocks = inputLen / 16 + 1;
	mod = inputLen % 16;
        if (mod == 0)
                numBlocks--;

	/* padding last block */
	p = outBuffer + (numBlocks - 1) * 16;
	for (i = 0; i < 16; i++)
		*p++ = mod;

	iv = IV;
	for (i = 0; i < numBlocks; i++) {
		memcpy(outBuffer, input, 16);
		for (k = 0; k < 128; k++) {
			if (mode) {
				rijndaelEncrypt(ctx->ek, ctx->Nr, iv, block);
				outBuffer[k >> 3] ^= (block[0] & 0x80U) >> (k & 7);
				for (t = 0; t < 15; t++) {
					iv[t] = (iv[t] << 1) | (iv[t + 1] >> 7);
				}
				iv[15] = (iv[15] << 1) | ((outBuffer[k >> 3] >> (7 - (k & 7))) & 1);
			} else {
				rijndaelEncrypt(ctx->ek, ctx->Nr, iv, block);
				for (t = 0; t < 15; t++) {
					iv[t] = (iv[t] << 1) | (iv[t + 1] >> 7);
				}
				iv[15] = (iv[15] << 1) | ((input[k >> 3] >> (7 - (k & 7))) & 1);
				outBuffer[k >> 3] ^= (block[0] & 0x80U) >> (k & 7);
			}
		}
		outBuffer += 16;
		input += 16;
	}

	return 16 * numBlocks;
}
